@@ -19,6 +19,7 @@ ffi = openrlib.ffi
 _options = ('rpy2', '--quiet', '--no-save')  # type: typing.Tuple[str, ...]
 logger.info('Default options to initialize R: {}'.format(', '.join(_options)))
 _DEFAULT_C_STACK_LIMIT: int = -1
+_DEFAULT_R_INTERACTIVE: bool = True
 rpy2_embeddedR_isinitialized = 0x00
 
 
@@ -155,9 +156,10 @@ def set_initoptions(options: typing.Tuple[str]) -> None:
     global _options
     for x in options:
         assert isinstance(x, str)
-    logger.info('Setting options to initialize R: {}'
-                .format(', '.join(options)))
-    _options = tuple(options)
+    with openrlib.rlock:
+        logger.info('Setting options to initialize R: {}'
+                    .format(', '.join(options)))
+        _options = tuple(options)
 
 
 def get_initoptions() -> typing.Tuple[str, ...]:
@@ -232,10 +234,24 @@ CALLBACK_INIT_PAIRS = (('ptr_R_WriteConsoleEx', '_consolewrite_ex'),
 
 # TODO: can init_once() be used here ?
 def _initr(
-        interactive: bool = True,
+        interactive: typing.Optional[bool] = None,
         _want_setcallbacks: bool = True,
-        _c_stack_limit: int = _DEFAULT_C_STACK_LIMIT
+        _c_stack_limit: typing.Optional[int] = None
 ) -> typing.Optional[int]:
+    """Initialize the embedded R.
+
+    :param interactive: Should R run in interactive or non-interactive mode?
+    if `None` the value in `_DEFAULT_R_INTERACTIVE` will be used.
+    :param _want_setcallbacks: Should custom rpy2 callbacks for R frontends
+    be set?.
+    :param _c_stack_limit: Limit for the C Stack.
+    if `None` the value in `_DEFAULT_C_STACK_LIMIT` will be used.
+    """
+
+    if interactive is None:
+        interactive = _DEFAULT_R_INTERACTIVE
+    if _c_stack_limit is None:
+        _c_stack_limit = _DEFAULT_C_STACK_LIMIT
 
     rlib = openrlib.rlib
     ffi_proxy = openrlib.ffi_proxy
