@@ -12,11 +12,12 @@ with warnings.catch_warnings():
                          on_conflict="warn",
                          version=dbplyr.__version__,
                          symbol_r2python=dbplyr._symbol_r2python,
-                         symbol_check_after=dbplyr._symbol_check_after)
-TARGET_VERSION = '1.2.1'
-if dbplyr.__version__ != TARGET_VERSION:
+                         symbol_resolve=dbplyr._symbol_resolve)
+TARGET_VERSION = '2.2.'
+if not dbplyr.__version__.startswith(TARGET_VERSION):
     warnings.warn(
-        'This was designed against dbplyr version %s '
-        'but you have %s' % (TARGET_VERSION, dbplyr.__version__))
+        'This was designed againt dbplyr versions starting with %s'
+        ' but you have %s' %
+        (TARGET_VERSION, dbplyr.__version__))
 
 sql = dbplyr.sql
